@@ -156,6 +156,8 @@ class REMI(MIDITokenizer):
                 current_tick += beat * time_division + pos * ticks_per_sample
                 current_bar = current_tick // ticks_per_bar
             elif event.type == 'Position':
+                if current_bar == -1:
+                    current_bar = 0  # as this Position token occurs before any Bar token
                 current_tick = current_bar * ticks_per_bar + int(event.value) * ticks_per_sample
             elif event.type == 'Tempo':
                 # If your encoding include tempo tokens, each Position token should be followed by
@@ -254,7 +256,7 @@ class REMI(MIDITokenizer):
             dic['Position'] += ['Chord']
 
         if self.additional_tokens['Tempo']:
-            dic['Tempo'] = ['Chord', 'Pitch']
+            dic['Tempo'] = ['Chord', 'Pitch'] if self.additional_tokens['Chord'] else ['Chord']
             dic['Position'] += ['Tempo']
 
         if self.additional_tokens['Rest']:
